# Author: Jonathan Prieto-Cubides (jonathan.cubides@uib.no)

import argparse
import glob
import re
import os
import pathlib

def remove_prefix(text, prefix):
  return re.sub(re.compile(prefix), '', text)

def findpostulates(regex, directory):
  postulates = 0
  outputfiles = {f : [] for f in sorted(glob.glob(f"{directory}**/*.lagda.md", recursive=True)) }
  for f in outputfiles.keys():
    with open(f) as _file:
      lines = _file.readlines()
      for i, line in enumerate(lines):
        if regex.search(line):
          lemma = []
          j = i
          while j < len(lines):
            sline = lines[j].lstrip()
            if len(sline) > 0 and not ("```" in sline):
              lemma += [sline]
              j += 1
            else:
              postulates += 1
              break

          lemma[0] = remove_prefix(lemma[0], "postulate(\s|\n){1,}")
          _lemma = ""
          colon = False
          sep = ""
          for line in lemma:
            px = line.lstrip()
            if px.startswith("→") or px.startswith(":"): sep = "  "
            _lemma += sep + line
          outputfiles[f] += ("(L{})".format(i),_lemma)

  print("No. postulates: ", postulates)
  print("-"*40)

  print("<pre class='Agda'>")
  for f, ans in outputfiles.items():
    if len(ans) > 0 :
      print("-"*80)
      print("{}:".format(f))
      for p in ans:
        print(p)
  print("</pre>")

REGEX ="^\s*postulate"
DIRECTORY="./../src/"

try:
    rgx = re.compile(REGEX)
except Exception as e:
    parser.error('Regex does not compile')
directory = pathlib.Path(DIRECTORY)
if not os.path.isdir(directory):
  parser.error('Directory is not valid')

findpostulates(rgx, DIRECTORY)